﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;

namespace WindowsGame1
{
    class XnaBoidsEffect
    {
        public Effect effect;

        // fx グローバル変数
        public EffectParameter g_WorldViewProj;

        public EffectParameter g_World;
        public EffectParameter g_World_inv;
        public EffectParameter g_ViewProj;
        public EffectParameter g_Light_dir;
        public EffectParameter g_color0;
        public EffectParameter g_color1;
        public EffectParameter g_Len;			

        // fx テクニック
        public EffectTechnique BasicTech;           // ボイド表示用 テクスチャ無し、ライト有り、頂点色使用
        public EffectTechnique TransformTech;       // 軌跡、衝突線球 用
        public EffectTechnique RenderSphereTech;    // 線球用


        private XnaBoidsEffect(Effect effect)
        {
            this.effect = effect;

            // These values change every frame, so we assign them
            // to EffectParameters to speed up access
            // Draw または Update 時にアクセスされる、エフェクトの各パラメータの EffectParameter インスタンスを作成します。
            // エフェクト パラメータには、Effect で Parameters のインデックス付きプロパティを使用してアクセスできます。
            // ただし、EffectParameters を使用した場合よりも遅くなります。
            // そのため、頻繁に変更される各エフェクト パラメータの EffectParameter を作成する必要があります。
            g_WorldViewProj = effect.Parameters["g_WorldViewProj"];

            g_World         = effect.Parameters["g_World"];
            g_World_inv     = effect.Parameters["g_World_inv"];
            g_ViewProj      = effect.Parameters["g_ViewProj"];
            g_Light_dir     = effect.Parameters["g_Light_dir"];
            g_color0        = effect.Parameters["g_color0"];
            g_color1        = effect.Parameters["g_color1"];
            g_Len           = effect.Parameters["g_Len"];

            BasicTech           = effect.Techniques["BasicTech"];       // ボイド表示用 テクスチャ無し、ライト有り、頂点色使用
            TransformTech       = effect.Techniques["TransformTech"];   // 軌跡、衝突線球 用
            RenderSphereTech    = effect.Techniques["RenderSphereTech"];// 線球用


            // 自発光色
            Vector4 emissive = new Vector4(0.1f, 0.1f, 0.1f, 1.0f);

            // 拡散反射色
            Vector4 diffuse = new Vector4(0.5f, 0.5f, 0.5f, 1.0f);

            effect.Parameters["g_color0"].SetValue(diffuse + emissive);	// 自発光色 + 拡散反射色

            // アンビエント
            Vector4 amb = new Vector4(0.5f, 0.5f, 0.5f, 1.0f);

            effect.Parameters["g_color1"].SetValue(amb);	// アンビエント


            //ライト方向 
            Vector4 light_dir = new Vector4(0.4285f, 0.7673f, 0.3430f, 0.0f);
            light_dir.Normalize();
            effect.Parameters["g_Light_dir"].SetValue(light_dir);

            /*
            アンビエント光 (環境光)
            アンビエント光は、光が拡散し、すべてのオブジェクトを一様に照らすライト。
            スペキュラ光 specular light 鏡面反射光
            物体に光があたったときに、最も強くその光を反射している部分の光の総称。
            デフューズ光 diffuse light 拡散反射光
            物体にあたった光はその内部に取り込まれ、反射を繰り返して拡散光となってふたたび外に出る。この拡散光の総称。
            */

        }


        // staticであることが重要!!
        public static XnaBoidsEffect LoadEffect(ContentManager content)
        {
            return new XnaBoidsEffect(content.Load<Effect>("XnaBoidsEffect"));
        }

    
    }
}
