﻿using System;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace ccc4
{

    // ========================================================================
    // unit球 unit表示時に unitAと一緒に表示する線の球
    class unit球 : 基本Model
    {
        VertexDeclaration       decl = null;

        // 線によるx,y,z座標軸+Unit表示
        VertexPositionColor[]   vpcUnit = null;
        VertexBuffer            vbUnit; // Vertex buffer to define a triangle.

        // --------------------------------------
        // 線による射程表示
        // 敵unitとの距離が射程内だったら円を表示
        // 1:射程近
        VertexPositionColor[] vpc射程1 = null;
        VertexBuffer            vb射程1;

        // 2:射程内
        VertexPositionColor[] vpc射程2 = null;
        VertexBuffer            vb射程2;

        線球 unit線球;

        // --------------------------------------
        public unit球()
        {}

        // --------------------------------------
        // size : 球の半径
        public void Initialize(GraphicsDeviceManager graphics, float 半径 )
        {
            float size = 半径;

            unit線球 = new 線球(
                                size,   // 半径
                                4,      // 円周分割数 
                                false,
                                Color.Green,        // 本体カラー
                                Color.DarkGreen);   // 赤道カラー


            this.unit線球.Initialize(graphics);

            decl = new VertexDeclaration(graphics.GraphicsDevice, VertexPositionColor.VertexElements);

            // ---------------------------
            // 線によるx,y,z座標軸+Unit表示 
            size = 半径 / 2f;   // unit球より小さく表示

            vbUnit = new VertexBuffer(graphics.GraphicsDevice, typeof(VertexPositionColor), 12, BufferUsage.WriteOnly );
            vpcUnit = new VertexPositionColor[12];
            {
                int i = 0;

                // x軸
                vpcUnit[i].Position = new Vector3(-size, 0,  0);   vpcUnit[i].Color = Color.Red;   i++;
                vpcUnit[i].Position = new Vector3( size, 0,  0);   vpcUnit[i].Color = Color.Red;   i++;

                // y軸 (-側の座標軸を取る)
                vpcUnit[i].Position = new Vector3( 0,    0, 0);   vpcUnit[i].Color = Color.Green;   i++;
                vpcUnit[i].Position = new Vector3( 0, size, 0);   vpcUnit[i].Color = Color.Green;   i++;

                // z軸
                vpcUnit[i].Position = new Vector3( 0, 0, -size);   vpcUnit[i].Color = Color.Blue;   i++;
                vpcUnit[i].Position = new Vector3( 0, 0,  size);   vpcUnit[i].Color = Color.Blue;   i++;

                // x軸 - z軸 アウトライン
                vpcUnit[i].Position = new Vector3(size, 0, 0);  vpcUnit[i].Color = Color.DarkSlateBlue; i++;
                vpcUnit[i].Position = new Vector3(0, 0, size); vpcUnit[i].Color = Color.DarkSlateBlue; i++;

                // y軸 - z軸 アウトライン
                vpcUnit[i].Position = new Vector3(0, size, 0); vpcUnit[i].Color = Color.DarkSlateBlue; i++;
                vpcUnit[i].Position = new Vector3(0, 0, size); vpcUnit[i].Color = Color.DarkSlateBlue; i++;

                // z軸 - x軸 アウトライン
                vpcUnit[i].Position = new Vector3(-size, 0, 0); vpcUnit[i].Color = Color.DarkSlateBlue; i++;
                vpcUnit[i].Position = new Vector3(0, 0, size); vpcUnit[i].Color = Color.DarkSlateBlue; i++;

                vbUnit.SetData<VertexPositionColor>(vpcUnit);
            }



            // ---------------------------
            size = 半径;

            // 線による射程表示
            // 1:射程近
            vb射程1 = new VertexBuffer(graphics.GraphicsDevice, typeof(VertexPositionColor), 33, BufferUsage.WriteOnly );
            
            vpc射程1 = new VertexPositionColor[33];
            {

                float 射程表示半径 = 1.5f;

                for (int i = 0; i < 33; i++)
                {
                    float sin = (float)Math.Sin(( Math.PI / 16) * i);
                    float cos = (float)Math.Cos(( Math.PI / 16) * i);

                    float x = 射程表示半径 * cos;
                    float y = 射程表示半径 * sin;

                    vpc射程1[i].Position = new Vector3( x, y, size);
                    vpc射程1[i].Color = Color.Yellow;
                }
                
                vb射程1.SetData<VertexPositionColor>(vpc射程1);
            }
            // ---------------------------
            // 線による射程表示
            // 2:射程内
            vb射程2 = new VertexBuffer(graphics.GraphicsDevice, typeof(VertexPositionColor), 33, BufferUsage.WriteOnly);
            vpc射程2 = new VertexPositionColor[33];
            {

                float 射程表示半径 = 1f;

                for (int i = 0; i < 33; i++)
                {
                    float sin = (float)Math.Sin((Math.PI / 16) * i);
                    float cos = (float)Math.Cos((Math.PI / 16) * i);

                    float x = 射程表示半径 * cos;
                    float y = 射程表示半径 * sin;

                    vpc射程2[i].Position = new Vector3(x, y, size);
                    vpc射程2[i].Color = Color.Blue;
                }
                vb射程2.SetData<VertexPositionColor>(vpc射程2);
            }

        }



        // --------------------------------------
        public void Draw2(  GraphicsDeviceManager graphics,
                            Effect effect2 // 隠れた線、色減衰
                         )
        {
            graphics.GraphicsDevice.VertexDeclaration = this.decl;

            this.unit線球.Draw2( graphics, effect2 );

            // --------------
            // 線によるx,y,z座標軸+Unit表示 : 色減衰有り(effect2)
            graphics.GraphicsDevice.Vertices[0].SetSource(vbUnit, 0, VertexPositionColor.SizeInBytes);

            effect2.Begin();
            foreach (EffectPass pass in effect2.CurrentTechnique.Passes)
            {
                pass.Begin();

                graphics.GraphicsDevice.DrawPrimitives(PrimitiveType.LineList, 0, 6);

                pass.End();
            }
            effect2.End();
        }




    } // unit球 end




    // ========================================================================
    public class 基本Model
    {
        private Matrix  m_単位行列  = Matrix.Identity;  // モデル単位行列
        private Vector3 m_pos       = Vector3.Zero;     // モデル位置

        // アクセサ ----------------------------------------------------------------------
        // 単位行列
        public Matrix 単位行列
        {
            get { return m_単位行列; }
            set { m_単位行列 = value; }
        }

        // 位置座標
        public Vector3 pos
        {
            get { return m_pos; }
            set { m_pos = value; }
        }

        public 基本Model()
		{
            m_単位行列 = Matrix.Identity;  // モデル単位行列
            m_pos = Vector3.Zero;     // モデル位置
        }

    }
}
